from typing import Any, Dict


class InitOptions:
    """A class to simplify the initialization of objects."""

    @classmethod
    def from_dict(cls, **kwargs: Dict[str, Any]):
        """Create instance from relevant keywords in dictionary.

        Instead of passing exactly the arguments defined by the class' __init__,
        this allows to pass a dictionary of many more values and the function will
        then only pass the correct arguments to the class' __init__.

        Example:
            >>> from valo_api_official.utils.init_options import InitOptions
            >>> class SomeClass(InitOptions):
            ...
            ...     def __init__(self, arg1, arg2):
            ...         self.arg1 = arg1
            ...         self.arg2 = arg2
            ...
            ...
            >>> many_args = {"arg1": 1, "arg2": 2, "arg3": 3}
            ... # This will raise an exception because __init__ does not expect arg3!
            >>> try:
            ...     SomeClass(**many_args)
            ... except TypeError:
            ...     pass
            ... # However this will work:
            >>> something = SomeClass.from_dict(**many_args)

        Args:
            **kwargs: Keyword arguments matching the constructor's variables.

        Returns:
            An instance of the class.
        """
        init_keys = (
            cls.__init__.__code__.co_varnames
        )  # Access the init functions arguments
        kwargs = {
            key: kwargs[key] for key in init_keys if key in kwargs
        }  # Select all elements in kwargs, that are also arguments of the init function
        return cls(**kwargs)



from typing import Dict, List, Optional

from dataclasses import dataclass

from valo_api_official.utils.init_options import InitOptions


@dataclass
class Entity(InitOptions):
    name: str
    id: str
    assetName: str
    assetPath: Optional[str] = None
    localizedNames: Optional[Dict[str, str]] = None


@dataclass
class Act(InitOptions):
    id: str
    parentId: str
    type: str
    name: str
    isActive: bool
    localizedNames: Optional[Dict[str, str]] = None


@dataclass
class ContentV1(InitOptions):
    version: str
    characters: List[Entity]
    maps: List[Entity]
    chromas: List[Entity]
    skins: List[Entity]
    skinLevels: List[Entity]
    equips: List[Entity]
    gameModes: List[Entity]
    sprays: List[Entity]
    sprayLevels: List[Entity]
    charms: List[Entity]
    charmLevels: List[Entity]
    playerCards: List[Entity]
    playerTitles: List[Entity]
    acts: List[Act]
    ceremonies: List[Entity]

    def __post_init__(self):
        self.characters = [Entity.from_dict(**c) for c in self.characters]
        self.maps = [Entity.from_dict(**m) for m in self.maps]
        self.chromas = [Entity.from_dict(**c) for c in self.chromas]
        self.skins = [Entity.from_dict(**s) for s in self.skins]
        self.skinLevels = [Entity.from_dict(**sl) for sl in self.skinLevels]
        self.equips = [Entity.from_dict(**e) for e in self.equips]
        self.gameModes = [Entity.from_dict(**gm) for gm in self.gameModes]
        self.sprays = [Entity.from_dict(**s) for s in self.sprays]
        self.sprayLevels = [Entity.from_dict(**sl) for sl in self.sprayLevels]
        self.charms = [Entity.from_dict(**c) for c in self.charms]
        self.charmLevels = [Entity.from_dict(**cl) for cl in self.charmLevels]
        self.playerCards = [Entity.from_dict(**pc) for pc in self.playerCards]
        self.playerTitles = [Entity.from_dict(**pt) for pt in self.playerTitles]
        self.acts = [Act.from_dict(**a) for a in self.acts]
        self.ceremonies = [Entity.from_dict(**c) for c in self.ceremonies]



from valo_api_official.config import Config


class EndpointsConfig:
    ACCOUNT_BY_PUUID = (
        f"https://{Config.NEAREST_CLUSTER}.api.riotgames.com/riot/account/"
        "{version}/accounts/by-puuid/{puuid}"
    )
    ACCOUNT_BY_NAME = (
        f"https://{Config.NEAREST_CLUSTER}.api.riotgames.com/riot/account/"
        "{version}/accounts/by-riot-id/{name}/{tag}"
    )

    ACTIVE_SHARDS = (
        f"https://{Config.NEAREST_CLUSTER}.api.riotgames.com/riot/account/"
        "{version}/active-shards/by-game/{game}/by-puuid/{puuid}"
    )

    CONTENT = (
        f"https://{Config.NEAREST_CLUSTER}.api.riotgames.com/val/content/"
        "{version}/contents"
    )

    STATUS = "https://{region}.api.riotgames.com/val/status/{version}/platform-data"
    LEADERBOARD = (
        "https://{region}.api.riotgames.com/val/ranked/{version}/leaderboards"
        "/by-act/{act_id}"
    )

    MATCH_DETAILS = (
        "https://{region}.api.riotgames.com/val/match/{version}/matches/{match_id}"
    )
    MATCH_HISTORY = "https://{region}.api.riotgames.com/val/match/{version}/matchlists/by-puuid/{puuid}"

    RAW = "/valorant/{version}/raw"
    STORE_FEATURED = "/valorant/{version}/store-featured"
    STORE_OFFERS = "/valorant/{version}/store-offers"
    VERSION_INFO = "/valorant/{version}/version/{region}"
    WEBSITE = "/valorant/{version}/website/{countrycode}"

    MMR_DETAILS_BY_PUUID = "/valorant/{version}/by-puuid/mmr/{region}/{puuid}"
    MMR_DETAILS_BY_NAME = "/valorant/{version}/mmr/{region}/{name}/{tag}"

    MMR_HISTORY_BY_PUUID = "/valorant/{version}/by-puuid/mmr-history/{region}/{puuid}"
    MMR_HISTORY_BY_NAME = "/valorant/{version}/mmr-history/{region}/{name}/{tag}"

    CROSSHAIR = "/valorant/{version}/crosshair/generate"



import os

from .endpoints import *


def set_api_key(api_key: str):
    """Sets the API key to use for all API requests.

    Args:
        api_key: The API key to use.
    """
    os.environ["VALO_API_OFFICIAL_KEY"] = api_key



from valo_api_official.endpoints.account import (
    get_account_by_name,
    get_account_by_name_v1,
    get_account_by_puuid,
    get_account_by_puuid_v1,
)
from valo_api_official.endpoints.active_shards import (
    get_active_shards,
    get_active_shards_v1,
)
from valo_api_official.endpoints.content import get_content, get_content_v1
from valo_api_official.endpoints.leaderboard import get_leaderboard, get_leaderboard_v1
from valo_api_official.endpoints.match_details import (
    get_match_details,
    get_match_details_v1,
)
from valo_api_official.endpoints.match_history import (
    get_match_history,
    get_match_history_v1,
)
from valo_api_official.endpoints.status import get_status, get_status_v1

__all__ = [
    "get_account_by_name_v1",
    "get_account_by_name",
    "get_account_by_puuid_v1",
    "get_account_by_puuid",
    "get_active_shards_v1",
    "get_active_shards",
    "get_content_v1",
    "get_content",
    "get_leaderboard_v1",
    "get_leaderboard",
    "get_match_details_v1",
    "get_match_details",
    "get_match_history_v1",
    "get_match_history",
    "get_status_v1",
    "get_status",
]



from typing import List, Optional

from dataclasses import dataclass

from valo_api_official.utils.init_options import InitOptions


@dataclass
class MatchInfo(InitOptions):
    matchId: str
    mapId: str
    gameVersion: str
    gameLengthMillis: int
    gameStartMillis: int
    isCompleted: bool
    customGameName: str
    gameMode: str
    isRanked: bool
    seasonId: str
    queueID: Optional[str] = None
    provisioningFlowID: Optional[str] = None


@dataclass
class PlayerAbilityCasts(InitOptions):
    grenadeCasts: int
    ability1Casts: int
    ability2Casts: int
    ultimateCasts: int


@dataclass
class PlayerStats(InitOptions):
    score: int
    roundsPlayed: int
    kills: int
    deaths: int
    assists: int
    playtimeMillis: int
    abilityCasts: Optional[PlayerAbilityCasts] = None

    def __post_init__(self):
        self.abilityCasts = (
            PlayerAbilityCasts.from_dict(**self.abilityCasts)
            if self.abilityCasts is not None
            else None
        )


@dataclass
class MatchPlayers(InitOptions):
    puuid: str
    gameName: str
    tagLine: str
    teamId: str
    partyId: str
    characterId: str
    stats: PlayerStats
    competitiveTier: int
    playerCard: str
    playerTitle: str

    def __post_init__(self):
        self.stats = PlayerStats.from_dict(**self.stats)


@dataclass
class MatchTeam(InitOptions):
    teamId: str
    won: bool
    roundsPlayed: int
    roundsWon: int
    numPoints: int


@dataclass
class Location(InitOptions):
    x: int
    y: int


@dataclass
class PlayerLocations(InitOptions):
    puuid: str
    viewRadians: float
    location: Location

    def __post_init__(self):
        self.location = Location.from_dict(**self.location)


@dataclass
class KillFinishingDamage(InitOptions):
    damageType: str
    damageItem: str
    isSecondaryFireMode: bool


@dataclass
class PlayerKills(InitOptions):
    timeSinceGameStartMillis: int
    timeSinceRoundStartMillis: int
    killer: str
    victim: str
    victimLocation: Location
    assistants: List[str]
    playerLocations: List[PlayerLocations]
    finishingDamage: KillFinishingDamage

    def __post_init__(self):
        self.victimLocation = Location.from_dict(**self.victimLocation)
        self.playerLocations = [
            PlayerLocations.from_dict(**x) for x in self.playerLocations
        ]
        self.finishingDamage = KillFinishingDamage.from_dict(**self.finishingDamage)


@dataclass
class PlayerDamage(InitOptions):
    receiver: str
    damage: int
    legshots: int
    bodyshots: int
    headshots: int


@dataclass
class PlayerEconomy(InitOptions):
    loadoutValue: int
    weapon: str
    armor: str
    remaining: int
    spent: int


@dataclass
class PlayerAbilityEffects(InitOptions):
    grenadeEffects: Optional[dict]
    ability1Effects: Optional[dict]
    ability2Effects: Optional[dict]
    ultimateEffects: Optional[dict]


@dataclass
class RoundPlayerStats(InitOptions):
    puuid: str
    kills: List[PlayerKills]
    damage: List[PlayerDamage]
    score: int
    economy: PlayerEconomy
    ability: PlayerAbilityEffects

    def __post_init__(self):
        self.kills = [PlayerKills.from_dict(**x) for x in self.kills]
        self.damage = [PlayerDamage.from_dict(**x) for x in self.damage]
        self.economy = PlayerEconomy.from_dict(**self.economy)
        self.ability = PlayerAbilityEffects.from_dict(**self.ability)


@dataclass
class MatchRoundResults(InitOptions):
    roundNum: int
    roundResult: str
    roundCeremony: str
    winningTeam: str
    bombPlanter: str
    bombDefuser: str
    plantRoundTime: int
    plantPlayerLocations: List[PlayerLocations]
    plantLocation: Location
    plantSite: str
    defuseRoundTime: int
    defusePlayerLocations: List[PlayerLocations]
    defuseLocation: Location
    playerStats: List[RoundPlayerStats]
    roundResultCode: str

    def __post_init__(self):
        self.plantPlayerLocations = [
            PlayerLocations.from_dict(**x) for x in self.plantPlayerLocations or []
        ]
        self.plantLocation = Location.from_dict(**self.plantLocation)
        self.defusePlayerLocations = [
            PlayerLocations.from_dict(**x) for x in self.defusePlayerLocations or []
        ]
        self.defuseLocation = Location.from_dict(**self.defuseLocation)
        self.playerStats = [RoundPlayerStats.from_dict(**x) for x in self.playerStats]


@dataclass
class MatchDetailsV1(InitOptions):
    matchInfo: MatchInfo
    players: List[MatchPlayers]
    coaches: List[dict]
    teams: List[MatchTeam]
    roundResults: Optional[List[MatchRoundResults]]

    def __post_init__(self):
        self.matchInfo = MatchInfo.from_dict(**self.matchInfo)
        self.players = [MatchPlayers.from_dict(**p) for p in self.players]
        self.teams = [MatchTeam.from_dict(**t) for t in self.teams]
        self.roundResults = (
            [MatchRoundResults.from_dict(**r) for r in self.roundResults]
            if self.roundResults is not None
            else None
        )



from valo_api_official.endpoints_config import EndpointsConfig
from valo_api_official.exceptions.valo_api_exception import ValoAPIException
from valo_api_official.responses.error_response import ErrorResponse
from valo_api_official.responses.status import StatusV1
from valo_api_official.utils.fetch_endpoint import fetch_endpoint


def get_status_v1(region: str, **kwargs) -> StatusV1:
    """Get the status for a specific region using version 1 of the endpoint.

    This is the same as
    :py:meth:`get_status(version="v1", region=region, **kwargs) <get_status>`

    Args:
        region: The region to get the status for.
            One of the following:
            eu (Europe), na (North America), ap (Asia Pacific), kr (Korea), latam (Latin America), br (Brazil)
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        StatusV1: Status fetched from the API.
    """
    return get_status("v1", region, **kwargs)


def get_status(version: str, region: str, **kwargs) -> StatusV1:
    """Get the status for a specific region.

    Args:
        version: The version of the endpoint to use.
            One of the following:
            v1 (Version 1)
        region: The region to get the status for.
            One of the following:
            eu (Europe), na (North America), ap (Asia Pacific), kr (Korea), latam (Latin America), br (Brazil)
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        StatusV1: Status fetched from the API.

    Raises:
        ValoAPIException: If the request failed.
    """
    response = fetch_endpoint(
        EndpointsConfig.STATUS,
        version=version,
        region=region,
        **kwargs,
    )
    response_data = response.json()

    if response.ok is False:
        headers = dict(response.headers)
        raise ValoAPIException(
            ErrorResponse.from_dict(
                headers=headers, **{"error": response_data["status"]}
            )
        )

    return StatusV1.from_dict(**response_data)



from typing import List, Optional, Union

from valo_api_official.exceptions.rate_limit import RateLimit
from valo_api_official.responses.error_response import Error, ErrorResponse


class ValoAPIException(Exception):
    """Base exception for all exceptions in the Valo API."""

    ratelimit: RateLimit

    def __init__(self, response: Union[ErrorResponse, str]):
        self.response = response
        self.ratelimit = RateLimit()
        super().__init__(response)

    def __str__(self):
        """
        Return the error message.

        Returns:
            The error message as string.
        """
        if not isinstance(self.response, ErrorResponse):
            return str(self.response)

        return self.response.error.message

    @property
    def status(self) -> Optional[str]:
        """
        Return the status code of the response.

        Returns:
            The status code of the response.
        """
        if not isinstance(self.response, ErrorResponse):
            return None
        return self.response.error.status_code

    @property
    def message(self) -> Optional[str]:
        """
        Return the error message of the response.

        Returns:
            The error message of the response.
        """
        if not isinstance(self.response, ErrorResponse):
            return None
        return self.response.error.message

    @property
    def errors(self) -> Optional[Error]:
        """
        Return the error code of the response.

        Returns:
            The error code of the response.
        """
        if not isinstance(self.response, ErrorResponse):
            return None
        return self.response.error



from typing import List, Optional

from dataclasses import dataclass

from valo_api_official.utils.init_options import InitOptions


@dataclass
class MaintenanceStatusTitleV1(InitOptions):
    locale: str
    content: str


@dataclass
class MaintenanceUpdateV1(InitOptions):
    publish_locations: List[str]
    updated_at: str
    created_at: str
    id: int
    publish: bool
    translations: List[MaintenanceStatusTitleV1]
    author: Optional[str] = None

    def __post_init__(self):
        self.translations = [MaintenanceStatusTitleV1(**t) for t in self.translations]


@dataclass
class MaintenancePointV1(InitOptions):
    updated_at: Optional[str]
    created_at: Optional[str]
    archive_at: Optional[str]
    titles: List[MaintenanceStatusTitleV1]
    maintenance_status: str
    id: int
    incident_severity: Optional[str]
    updates: List[MaintenanceUpdateV1]
    platforms: List[str]

    def __post_init__(self):
        self.titles = [MaintenanceStatusTitleV1.from_dict(**t) for t in self.titles]
        self.updates = [MaintenanceUpdateV1.from_dict(**u) for u in self.updates]


@dataclass
class StatusV1(InitOptions):
    id: str
    name: str
    locales: List[str]
    maintenances: List[MaintenancePointV1]
    incidents: List[MaintenancePointV1]

    def __post_init__(self):
        self.maintenances = [
            MaintenancePointV1.from_dict(**m) for m in self.maintenances
        ]
        self.incidents = [MaintenancePointV1.from_dict(**i) for i in self.incidents]



from typing import List

from dataclasses import dataclass

from valo_api_official.utils.init_options import InitOptions


@dataclass
class MatchRaw(InitOptions):
    matchId: str
    gameStartTimeMillis: int
    queueId: str


@dataclass
class MatchHistoryV1(InitOptions):
    puuid: str
    history: List[MatchRaw]

    def __post_init__(self):
        self.history = [MatchRaw.from_dict(**h) for h in self.history]



from typing import Optional

from valo_api_official.endpoints_config import EndpointsConfig
from valo_api_official.exceptions.valo_api_exception import ValoAPIException
from valo_api_official.responses.error_response import ErrorResponse
from valo_api_official.responses.leaderboard import LeaderboardV1
from valo_api_official.utils.fetch_endpoint import fetch_endpoint


def get_leaderboard_v1(
    region: str,
    act_id: Optional[str] = None,
    start: Optional[int] = None,
    size: Optional[int] = None,
    **kwargs,
) -> LeaderboardV1:
    """Get the leaderboard for a region using version 1 of the endpoint.

    This is the same as
    :py:meth:`get_leaderboard(version="v1", region=region, puuid=puuid, name=name, tag=tag, **kwargs) <get_leaderboard>`

    You can also filter by puuid or name and tag.

    Args:
        region: The region to get the leaderboard for.
            One of the following:
            eu (Europe), na (North America), ap (Asia Pacific), kr (Korea), latam (Latin America), br (Brazil)
        act_id: The season ID to get the leaderboard for.
        start: The start index of the leaderboard. Can be used for pagination.
        size: The size of the leaderboard.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        LeaderboardV1: The leaderboard for the region.
    """
    return get_leaderboard("v1", region, act_id, start, size, **kwargs)


def get_leaderboard(
    version: str,
    region: str,
    act_id: Optional[str] = None,
    start: Optional[int] = None,
    size: Optional[int] = None,
    **kwargs,
) -> LeaderboardV1:
    """Get the leaderboard for a region using a specific version of the endpoint.

    You can also filter by puuid or name and tag if you are using v1.

    Args:
        version: The version of the endpoint. One of the following:
            v1 (Version 1)
        region: The region to get the leaderboard for.
            One of the following:
            eu (Europe), na (North America), ap (Asia Pacific), kr (Korea), latam (Latin America), br (Brazil)
        act_id: The act ID to get the leaderboard for.
        start: The start index of the leaderboard. Can be used for pagination.
        size: The size of the leaderboard.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        LeaderboardV1: The leaderboard for the region.

    Raises:
        ValoAPIException: If the request failed.
    """
    query_args = dict()
    if start is not None:
        query_args["start"] = start
    if size is not None:
        query_args["size"] = start

    response = fetch_endpoint(
        EndpointsConfig.LEADERBOARD,
        region=region,
        version=version,
        act_id=act_id,
        query_args=query_args,
        **kwargs,
    )
    response_data = response.json()

    if response.ok is False:
        headers = dict(response.headers)
        raise ValoAPIException(
            ErrorResponse.from_dict(
                headers=headers, **{"error": response_data["status"]}
            )
        )

    return LeaderboardV1.from_dict(**response_data)



from typing import Dict, List, Optional

from dataclasses import dataclass

from valo_api_official.utils.init_options import InitOptions


@dataclass
class LeaderboardPlayerV1(InitOptions):
    leaderboardRank: int
    rankedRating: int
    numberOfWins: int
    competitiveTier: int
    puuid: Optional[str] = None
    gameName: Optional[str] = None
    tagLine: Optional[str] = None


@dataclass
class TierDetail(InitOptions):
    rankedRatingThreshold: int
    startingPage: int
    startingIndex: int


@dataclass
class LeaderboardV1(InitOptions):
    actId: str
    players: List[LeaderboardPlayerV1]
    totalPlayers: int
    immortalStartingPage: int
    immortalStartingIndex: int
    topTierRRThreshold: int
    tierDetails: Dict[int, TierDetail]
    startIndex: int
    query: str
    shard: str

    def __post_init__(self):
        self.players = [
            LeaderboardPlayerV1.from_dict(**player)
            for player in self.players
            if player is not None
        ]
        self.tierDetails = {
            tier: TierDetail.from_dict(**details)
            for tier, details in self.tierDetails.items()
        }



from valo_api_official.endpoints_config import EndpointsConfig
from valo_api_official.exceptions.valo_api_exception import ValoAPIException
from valo_api_official.responses.account import AccountV1
from valo_api_official.responses.error_response import ErrorResponse
from valo_api_official.utils.fetch_endpoint import fetch_endpoint


def get_account_by_name_v1(name: str, tag: str, **kwargs) -> AccountV1:
    """Get the account details for a player using version 1 of the endpoint.

    This is the same as
    :py:meth:`get_account_details_by_name(version="v1", name=name, tag=tag, **kwargs) <get_account_details_by_name>`

    Args:
        name: The name of the player to get the account details for.
        tag: The tag of the player to get the account details for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        AccountV1: Account details fetched from the API.
    """
    return get_account_by_name("v1", name, tag, **kwargs)


def get_account_by_name(version: str, name: str, tag: str, **kwargs) -> AccountV1:
    """Get the account details for a player using a specific version of the endpoint.

    Args:
        version: The version of the endpoint to use.
            One of the following:
            v1 (Version 1)
        name: The name of the player to get the account details for.
        tag: The tag of the player to get the account details for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        AccountV1: Account details fetched from the API.

    Raises:
        ValoAPIException: If the request failed.
    """
    response = fetch_endpoint(
        EndpointsConfig.ACCOUNT_BY_NAME,
        version=version,
        name=name,
        tag=tag,
        **kwargs,
    )
    response_data = response.json()

    if response.ok is False:
        headers = dict(response.headers)
        raise ValoAPIException(
            ErrorResponse.from_dict(
                headers=headers, **{"error": response_data["status"]}
            )
        )

    return AccountV1.from_dict(**response_data)


def get_account_by_puuid_v1(puuid: str, **kwargs) -> AccountV1:
    """Get the account details for a player using version 1 of the endpoint.

    This is the same as
    :py:meth:`get_account_details_by_puuid(version="v1", puuid=puuid, **kwargs) <get_account_details_by_puuid>`

    Args:
        puuid: The PUUID of the player to get the account details for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        AccountV1: Account details fetched from the API.
    """
    return get_account_by_puuid("v1", puuid, **kwargs)


def get_account_by_puuid(version: str, puuid: str, **kwargs) -> AccountV1:
    """Get the account details for a player using a specific version of the endpoint.

    Args:
        version: The version of the endpoint to use.
            One of the following:
            v1 (Version 1)
        puuid: The PUUID of the player to get the account details for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        AccountV1: Account details fetched from the API.

    Raises:
        ValoAPIException: If the request failed.
    """
    response = fetch_endpoint(
        EndpointsConfig.ACCOUNT_BY_PUUID,
        version=version,
        puuid=puuid,
        **kwargs,
    )
    response_data = response.json()

    if response.ok is False:
        headers = dict(response.headers)
        raise ValoAPIException(
            ErrorResponse.from_dict(
                headers=headers, **{"error": response_data["status"]}
            )
        )

    return AccountV1.from_dict(**response_data)



from typing import Any, Dict, Optional

import os
import urllib.parse

import requests
from requests import Response

from valo_api_official.config import Config
from valo_api_official.exceptions.rate_limit import RateLimit


def encode_params(**kwargs) -> Dict[str, str]:
    """Returns a string of the parameters to be used in a URL.

    Args:
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        A dictionary of the parameters to be used in a URL.
    """
    out = dict()
    for key, value in kwargs.items():
        encoded_key = urllib.parse.quote_plus(str(key))
        encoded_value = urllib.parse.quote_plus(str(value))
        out[encoded_key] = encoded_value
    return out


def fetch_endpoint(
    url: str,
    query_args: Optional[Dict[str, Any]] = None,
    method: str = "GET",
    **kwargs,
) -> Response:
    """Fetches an endpoint from the API.

    Args:
        url: The endpoint definition to use.
        query_args: Any additional arguments to pass to the endpoint.
        method: The method to use when fetching the endpoint.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        A response from the API.
    """
    fetch_endpoint.session = (
        fetch_endpoint.session
        if hasattr(fetch_endpoint, "session")
        else requests.Session()
    )

    url = url.lower()
    encoded_params = encode_params(**kwargs)

    # Build the URL
    # First Replace the parameters in the endpoint definition
    for key, value in encoded_params.items():
        url = url.replace(f"{{{key.lower()}}}", value.lower())

    # Set the headers
    headers = {
        "User-Agent": Config.USER_AGENT,
        "Accept": "application/json",
    }
    if (
        "VALO_API_OFFICIAL_KEY" in os.environ
        and os.environ["VALO_API_OFFICIAL_KEY"] is not None
    ):
        headers["X-Riot-Token"] = os.environ["VALO_API_OFFICIAL_KEY"]

    # Make the request
    response = fetch_endpoint.session.request(
        method, url, params=query_args, json=query_args, headers=headers
    )
    RateLimit.limit, RateLimit.count = (
        response.headers.get("X-Method-Rate-Limit", -1),
        response.headers.get("X-Method-Rate-Limit-Count", -1),
    )

    return response



from valo_api_official.endpoints_config import EndpointsConfig
from valo_api_official.exceptions.valo_api_exception import ValoAPIException
from valo_api_official.responses.error_response import ErrorResponse
from valo_api_official.responses.match_details import MatchDetailsV1
from valo_api_official.utils.fetch_endpoint import fetch_endpoint


def get_match_details_v1(region: str, match_id: str, **kwargs) -> MatchDetailsV1:
    """Get the match details for a match using version 1 of the endpoint.

    This is the same as
    :py:meth:`get_match_details(version="v1", matchId=matchId, **kwargs) <get_match_details>`

    Args:
        region: The region to get the leaderboard for.
            One of the following:
            eu (Europe), na (North America), ap (Asia Pacific), kr (Korea), latam (Latin America), br (Brazil)
        match_id: The match ID to get the details for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        MatchDetailsV1: Match details fetched from the API.
    """
    return get_match_details("v1", region, match_id, **kwargs)


def get_match_details(
    version: str, region: str, match_id: str, **kwargs
) -> MatchDetailsV1:
    """Get the match details for a match using a specific version of the endpoint.

    Args:
        version: The version of the endpoint to use.
            One of the following:
            v1 (Version 1)
        region: The region to get the leaderboard for.
            One of the following:
            eu (Europe), na (North America), ap (Asia Pacific), kr (Korea), latam (Latin America), br (Brazil)
        match_id: The match ID to get the details for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        MatchDetailsV1: Match details fetched from the API.

    Raises:
        ValoAPIException: If the request failed.
    """
    response = fetch_endpoint(
        EndpointsConfig.MATCH_DETAILS,
        version=version,
        region=region,
        match_id=match_id,
        **kwargs,
    )
    response_data = response.json()

    if response.ok is False:
        headers = dict(response.headers)
        raise ValoAPIException(
            ErrorResponse.from_dict(
                headers=headers, **{"error": response_data["status"]}
            )
        )

    return MatchDetailsV1.from_dict(**response_data)



class RateLimit:
    limit: str
    """The number of requests you can make in the current period."""
    count: str
    """The number of requests you did in the current period."""

    def __str__(self) -> str:
        return f"Limit: {self.limit}; Count: {self.count}"


def rate_limit() -> RateLimit:
    """Returns the current rate limit for the API.

    Returns:
        RateLimit: A :class:`.RateLimit` object.
    """
    return RateLimit()



from typing import Optional

from valo_api_official.endpoints_config import EndpointsConfig
from valo_api_official.exceptions.valo_api_exception import ValoAPIException
from valo_api_official.responses.content import ContentV1
from valo_api_official.responses.error_response import ErrorResponse
from valo_api_official.utils.fetch_endpoint import fetch_endpoint


def get_content_v1(locale: Optional[str] = None, **kwargs) -> ContentV1:
    """Get the content for a specific locale using version 1 of the endpoint.

    This is the same as
    :py:meth:`get_content(version="v1", locale=locale, **kwargs) <get_content>`

    Args:
        locale: The locale to get the content for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        ContentV1: Content fetched from the API.
    """
    return get_content("v1", locale, **kwargs)


def get_content(version: str, locale: Optional[str] = None, **kwargs) -> ContentV1:
    """Get the content for a specific locale using a specific version of the endpoint.

    Args:
        version: The version of the endpoint to use.
            One of the following:
            v1 (Version 1)
        locale: The locale to get the content for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        ContentV1: Content fetched from the API.

    Raises:
        ValoAPIException: If the request failed.
    """
    response = fetch_endpoint(
        EndpointsConfig.CONTENT,
        version=version,
        query_args={"locale": str(locale).lower()} if locale else None,
        **kwargs,
    )
    response_data = response.json()

    if response.ok is False:
        headers = dict(response.headers)
        raise ValoAPIException(
            ErrorResponse.from_dict(
                headers=headers, **{"error": response_data["status"]}
            )
        )

    return ContentV1.from_dict(**response_data)



from typing import List, Optional

from dataclasses import dataclass

from valo_api_official.utils.init_options import InitOptions


@dataclass
class Error(InitOptions):
    message: Optional[str] = None
    status_code: Optional[int] = None


@dataclass
class ErrorResponse(InitOptions):
    headers: Optional[dict] = None
    error: Optional[Error] = None

    def __post_init__(self):
        self.error = Error.from_dict(**self.error) if self.error is not None else None



from valo_api_official.endpoints_config import EndpointsConfig
from valo_api_official.exceptions.valo_api_exception import ValoAPIException
from valo_api_official.responses.active_shards import ActiveShardsV1
from valo_api_official.responses.error_response import ErrorResponse
from valo_api_official.utils.fetch_endpoint import fetch_endpoint


def get_active_shards_v1(puuid: str, **kwargs) -> ActiveShardsV1:
    """Get the active shards for a player using version 1 of the endpoint.

    This is the same as
    :py:meth:`get_active_shards(version="v1", puuid=puuid, **kwargs) <get_active_shards>`

    Args:
        puuid: The PUUID of the player to get the active shards for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        ActiveShardsV1: Active Shards fetched from the API.
    """
    return get_active_shards("v1", puuid, **kwargs)


def get_active_shards(version: str, puuid: str, **kwargs) -> ActiveShardsV1:
    """Get the active shards for a player using a specific version of the endpoint.

    Args:
        version: The version of the endpoint to use.
            One of the following:
            v1 (Version 1)
        puuid: The PUUID of the player to get the active shards for.
        **kwargs: Any additional arguments to pass to the endpoint.

    Returns:
        ActiveShardsV1: Active Shards fetched from the API.

    Raises:
        ValoAPIException: If the request failed.
    """
    response = fetch_endpoint(
        EndpointsConfig.ACTIVE_SHARDS,
        version=version,
        puuid=puuid,
        game="val",
        **kwargs,
    )
    response_data = response.json()

    if response.ok is False:
        headers = dict(response.headers)
        raise ValoAPIException(
            ErrorResponse.from_dict(
                headers=headers, **{"error": response_data["status"]}
            )
        )

    return ActiveShardsV1.from_dict(**response_data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['valo_api_official',
 'valo_api_official.endpoints',
 'valo_api_official.exceptions',
 'valo_api_official.responses',
 'valo_api_official.utils']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'valo-api-official',
    'version': '0.0.5',
    'description': 'Valorant API Wrapper',
    'long_description': '# valo_api_official\n\n<div align="center">\n\n[![Build status](https://github.com/raimannma/ValorantAPIOfficial/workflows/build/badge.svg?branch=master&event=push)](https://github.com/raimannma/ValorantAPIOfficial/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/valo_api_official.svg)](https://pypi.org/project/valo_api_official/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/raimannma/ValorantAPI/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/raimannma/ValorantAPIOfficial/blob/master/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/raimannma/ValorantAPIOfficial/releases)\n[![License](https://img.shields.io/github/license/raimannma/ValorantAPIOfficial)](https://github.com/raimannma/ValorantAPIOfficial/blob/master/LICENSE)\n[![Codacy Badge](https://app.codacy.com/project/badge/Grade/3b23d2a3b1694356bc95255a2edb83e6)](https://www.codacy.com/gh/raimannma/ValorantAPIOfficial/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=raimannma/ValorantAPIOfficial&amp;utm_campaign=Badge_Grade)\n[![Codacy Badge](https://app.codacy.com/project/badge/Coverage/3b23d2a3b1694356bc95255a2edb83e6)](https://www.codacy.com/gh/raimannma/ValorantAPIOfficial/dashboard?utm_source=github.com&utm_medium=referral&utm_content=raimannma/ValorantAPIOfficial&utm_campaign=Badge_Coverage)\n[![Downloads](https://pepy.tech/badge/valo-api-official)](https://pepy.tech/project/valo-api-official)\n\nValorant API Wrapper\n\n</div>\n\n## Installation\n\n    pip install valo-api-official\n\n## Documentation\n\n### Hosted\n\nThe documentation is hosted here: https://raimannma.github.io/ValorantAPIOfficial/\n\n### From Source\n\nAfter installing the package dependencies `pip install -r requirements.txt`, you can use the following commands to get the documentation:\n\n    cd docs/ && make html\n\nOpen the index.html file in the docs/_build/html/ directory.\n',
    'author': 'Manuel Raimann',
    'author_email': 'raimannma@outlook.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/raimannma/ValorantAPIOfficial',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import List


class Config:
    NEAREST_CLUSTER = "europe"
    USER_AGENT: str = "Python Valorant API Wrapper"
    BASE_URL: str = "https://api.henrikdev.xyz"
    ALL_REGIONS: List[str] = ["eu", "na", "ap", "kr", "latam", "br"]
    ALL_COUNTRY_CODES = [
        "en-us",
        "en-gb",
        "de-de",
        "es-es",
        "fr-fr",
        "it-it",
        "ru-ru",
        "tr-tr",
        "es-mx",
        "ja-jp",
        "ko-kr",
        "pt-br",
        "vi-vn",
    ]
    ALL_LOCALS = [
        "ar-AE",
        "de-DE",
        "en-US",
        "es-ES",
        "es-MX",
        "fr-FR",
        "id-ID",
        "it-IT",
        "ja-JP",
        "ko-KR",
        "pl-PL",
        "pt-BR",
        "ru-RU",
        "th-TH",
        "tr-TR",
        "vi-VN",
        "zh-CN",
        "zh-TW",
    ]
